
#include <WiServer.h>

#define WIRELESS_MODE_INFRA	1
#define WIRELESS_MODE_ADHOC	2



// YellowJacket + Arduino reseved pins
// SPI: 10,11,12,13
// Connections status led: 9
// Wifi enable (interrupt): 8
// Interrupt: 2




// Wireless configuration parameters ----------------------------------------
unsigned char local_ip[] = {
  192,168,0,240};	// IP address of WiShield
unsigned char gateway_ip[] = {
  192,168,0,3};	// router or gateway IP address
unsigned char subnet_mask[] = {
  255,255,255,0};	// subnet mask for the local network
const prog_char ssid[] PROGMEM = {
  "{SUA REDE}"};		// max 32 bytes

unsigned char security_type = 3;	// 0 - open; 1 - WEP; 2 - WPA; 3 - WPA2

// WPA/WPA2 passphrase
const prog_char security_passphrase[] PROGMEM = {
  "{SUA SENHA}"};	// max 64 characters

// WEP 128-bit keys
// sample HEX keys
prog_uchar wep_keys[] PROGMEM = { 
  0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d,	// Key 0
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Key 1
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Key 2
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00	// Key 3
};

// setup the wireless mode
// infrastructure - connect to Access Point
// adhoc - connect to another WiFi device
unsigned char wireless_mode = WIRELESS_MODE_INFRA;

unsigned char ssid_len;
unsigned char security_passphrase_len;
// End of wireless configuration parameters ----------------------------------------









boolean varLed = false;
boolean flagRelay = false;
boolean reedState = false;

boolean elevatorIsHere = false;
boolean elevatorIsComing = false;
boolean reedSwitchFlag = false;

boolean flagIntervalReedSwitch;
boolean flagCheckConnection =false;

int ledPin1 = 7;
int ledPin2 = 4;
int relePin = 6;
int reedPin = 5;

int pinReset = A5;

long relayInterval = 400; // 400 millis = .4 seconds
long reedSwitchInterval = 7000; // 7000 millis = 7 seconds

int counterRele = 0;
int counterReedSwitch = 0;

int serverUp = 0;
long updateTimePing = 0;
unsigned long previousMillis;
unsigned long previousMillisRele;
unsigned long previousMillisReed;

int countTimer;

GETrequest pingRouter(gateway_ip, 80, "www.google.com", "/");









void setup() {
	Serial.begin(57600);
	Serial.println("start!");
  	//Enable Serial output and ask WiServer to generate log messages (optional)
  	//WiServer.enableVerboseMode(true);
	
	//pin config
  	pinMode(reedPin, INPUT); 
  	pinMode(relePin, OUTPUT); 
  	pinMode(ledPin1, OUTPUT); 
    pinMode(ledPin2, OUTPUT); 
  	pinMode(pinReset, OUTPUT); 

  	digitalWrite(pinReset, HIGH); 
  	pingRouter.setReturnFunc(printData);
	ledWhite();

}















void loop(){
	// reads the state of the reed switch every loop cycle
	reedState = digitalRead(reedPin);
	
	// debug - lights the led when the reed switch is triggered
	if(reedState == 1) {
		ledGreen();
	} else {
		ledWhite();  
	}
	
	// 1 second interval routine
	if(millis() - previousMillis > 1000) {
	    previousMillis = millis();
		countTimer++;
		Serial.print(countTimer);
		Serial.print(" Connection Status > ");
		Serial.println(WiServer.connection_up());
		
		// 90 seconds interval
		if(countTimer==90){
			flagCheckConnection = true;
			pingRouter.submit();  
			countTimer = 0;
		}
	}
	
	
	
	//interval routine after triggering the relay
	if(flagRelay) {
		if(millis() - previousMillisRele > relayInterval) {
		    previousMillisRele = millis();
			deactivateRelay();
			reedSwitchFlag = true;
		}
	}
	
	// timer needed to read the reed switch in case the elevator goes up the current floor
	if(reedSwitchFlag) {
		if(reedState == 1) {
			// 9 seconds interval to switch the reed switch flag
			if(millis() - previousMillisReed > reedSwitchInterval) { 
			    previousMillisReed = millis();
				flagIntervalReedSwitch = true;
				elevatorIsHereFunction();
			}
		} else {
			previousMillisReed = millis();
			flagIntervalReedSwitch = false;
		}	
	}
	
   // flag connection
   if(WiServer.connection_up() == 1) {
	flagCheckConnection = true;
   }

   if(flagCheckConnection) {
		checkConnection();
   }
   

   if(!serverUp) {
     serverUp = WiServer.async_init(sendMyPage, 0);         
   } else {
     WiServer.server_task();
   }
	
  //delay(10);
}














void checkConnection() {
	 if (WiServer.connection_up()) {
	    // life is good, no worries
	 } else {   
		// life sucks, reset!
	    serverUp = WiServer.async_init(sendMyPage, 1);
		resetArduino();
	 }
}

void callElevator() {
	if(elevatorIsComing == false) {
		if(reedState == 1) {
			elevatorIsHereFunction();
		} else {
			activateRelay(); 
		}
	}
}

void forceCallElevator() {
	digitalWrite(relePin, HIGH);
	delay(300);
	digitalWrite(relePin, LOW);
}

void activateRelay() {
  	digitalWrite(relePin, HIGH);
	previousMillisRele = millis();
 	flagRelay = true;
	elevatorIsHere = false;
	elevatorIsComing = true;
}

void deactivateRelay() {
  	digitalWrite(relePin, LOW);
	flagRelay = false;
}

void elevatorIsHereFunction() {
  	reedSwitchFlag = false;
  	elevatorIsHere = true;
	elevatorIsComing = false;
  	Serial.println("------------------");
  	Serial.println("Elevator is Here!");
  	Serial.println("------------------");
}

void resetArduino() {
  digitalWrite(pinReset, LOW);
}


// This is our page serving function that generates web pages
boolean sendMyPage(char* URL) {
	
  // reset Arduino URL
  if (strcmp(URL, "/reset") == 0) {
	WiServer.print("reset=1");
    resetArduino();
    return true;
  }
	
	

	
  // reset Arduino vars URL
  if (strcmp(URL, "/resetVars") == 0) {
	varLed = false;
	flagRelay = false;
	reedState = false;
	elevatorIsHere = false;
	elevatorIsComing = false;
	reedSwitchFlag = false;
	flagIntervalReedSwitch = false;
	
    return true;
  }




  // force light LED
  if (strcmp(URL, "/led") == 0) {
    varLed = !varLed;
    digitalWrite(ledPin1, varLed); 
    return true;
  }




  // call elevator URL
  if (strcmp(URL, "/call") == 0) {
      callElevator();
      // WiServer.print("<html>");
      WiServer.print("call=1");
      // WiServer.print("</html>");
    return true;
  }




  // force relay URL
  if (strcmp(URL, "/forceCall") == 0) {
      forceCallElevator();
      // WiServer.print("<html>");
      WiServer.print("forceCall=1");
      // WiServer.print("</html>");
    return true;
  }



  // variables status URL
  if (strcmp(URL, "/") == 0) {
    //WiServer.print("<html>");
    //WiServer.print("<br>The arduino has been running for: "); 
    //WiServer.print(millis()); 
    //WiServer.print(" milliseconds<br><br>"); 
	
	if(reedSwitchFlag) {
		if(reedState == 1 && flagIntervalReedSwitch) {
			elevatorIsHere = true;
		} else {
			elevatorIsHere = false;
		}
	} else {
		if(reedState == 1) {
			elevatorIsHere = true;
		} else {
			elevatorIsHere = false;
		}
	}
	
    WiServer.print("elevatorIsHere=");
    WiServer.print(elevatorIsHere); 
	WiServer.print("&"); 
	WiServer.print("elevatorIsComing=");
    WiServer.print(elevatorIsComing);
    //WiServer.print("</html>");
    return true;
  }

  // URL not found
  return false;
}



void ledGreen() {
	digitalWrite(ledPin1, HIGH); 
	digitalWrite(ledPin2, LOW); 	
}

void ledWhite() {
	digitalWrite(ledPin1, LOW); 
	digitalWrite(ledPin2, HIGH);
}

void ledsOff() {
	digitalWrite(ledPin1, LOW); 
	digitalWrite(ledPin2, LOW);
}

// Function that prints data from the server
void printData(char* data, int len) {
  // Print the data returned by the server
  // Note that the data is not null-terminated, may be broken up into smaller packets, and 
  // includes the HTTP header. 
  while (len-- > 0) {
    Serial.print(*(data++));
  } 
}


/*
void flash() {
  static boolean output = HIGH;
  digitalWrite(ledPin, output);
  output = !output;
	Serial.println(output);
}
*/